diff --git a/src/libFLAC/stream_encoder_intrin_sse2.c b/src/libFLAC/stream_encoder_intrin_sse2.c
index 9170194..be026a1 100644
--- a/src/libFLAC/stream_encoder_intrin_sse2.c
+++ b/src/libFLAC/stream_encoder_intrin_sse2.c
@@ -59,11 +59,9 @@ void FLAC__precompute_partition_info_sums_intrin_sse2(const FLAC__int32 residual
 		__m128i mm_res, mm_sum, mm_mask;
 
 		if(bps <= 16) {
-			FLAC__uint32 abs_residual_partition_sum;
 
 			for(partition = residual_sample = 0; partition < partitions; partition++) {
 				end += default_partition_samples;
-				abs_residual_partition_sum = 0;
 				mm_sum = _mm_setzero_si128();
 
 				e1 = (residual_sample + 3) & ~3; e3 = end & ~3;
@@ -71,27 +69,33 @@ void FLAC__precompute_partition_info_sums_intrin_sse2(const FLAC__int32 residual
 					e1 = end; /* try flac -l 1 -b 16 and you'll be here */
 
 				/* assumption: residual[] is properly aligned so (residual + e1) is properly aligned too and _mm_loadu_si128() is fast */
-				for( ; residual_sample < e1; residual_sample++)
-					abs_residual_partition_sum += abs(residual[residual_sample]); /* abs(INT_MIN) is undefined, but if the residual is INT_MIN we have bigger problems */
+				for( ; residual_sample < e1; residual_sample++) {
+					mm_res = _mm_cvtsi32_si128(residual[residual_sample]);
+					mm_mask = _mm_srai_epi32(mm_res, 31);
+					mm_res = _mm_xor_si128(mm_res, mm_mask);
+					mm_res = _mm_sub_epi32(mm_res, mm_mask); /* abs(INT_MIN) is undefined, but if the residual is INT_MIN we have bigger problems */
+					mm_sum = _mm_add_epi32(mm_sum, mm_res);
+				}
 
 				for( ; residual_sample < e3; residual_sample+=4) {
 					mm_res = _mm_loadu_si128((const __m128i*)(residual+residual_sample));
-
 					mm_mask = _mm_srai_epi32(mm_res, 31);
 					mm_res = _mm_xor_si128(mm_res, mm_mask);
 					mm_res = _mm_sub_epi32(mm_res, mm_mask);
+					mm_sum = _mm_add_epi32(mm_sum, mm_res);
+				}
 
+				for( ; residual_sample < end; residual_sample++) {
+					mm_res = _mm_cvtsi32_si128(residual[residual_sample]);
+					mm_mask = _mm_srai_epi32(mm_res, 31);
+					mm_res = _mm_xor_si128(mm_res, mm_mask);
+					mm_res = _mm_sub_epi32(mm_res, mm_mask);
 					mm_sum = _mm_add_epi32(mm_sum, mm_res);
 				}
 
 				mm_sum = _mm_add_epi32(mm_sum, _mm_srli_si128(mm_sum, 8));
 				mm_sum = _mm_add_epi32(mm_sum, _mm_srli_si128(mm_sum, 4));
-				abs_residual_partition_sum += _mm_cvtsi128_si32(mm_sum);
-
-				for( ; residual_sample < end; residual_sample++)
-					abs_residual_partition_sum += abs(residual[residual_sample]);
-
-				abs_residual_partition_sums[partition] = abs_residual_partition_sum;
+				abs_residual_partition_sums[partition] = _mm_cvtsi128_si32(mm_sum);
 			}
 		}
 		else { /* have to pessimistically use 64 bits for accumulator */
diff --git a/src/libFLAC/stream_encoder_intrin_ssse3.c b/src/libFLAC/stream_encoder_intrin_ssse3.c
index d68689e..4984b0a 100644
--- a/src/libFLAC/stream_encoder_intrin_ssse3.c
+++ b/src/libFLAC/stream_encoder_intrin_ssse3.c
@@ -59,11 +59,9 @@ void FLAC__precompute_partition_info_sums_intrin_ssse3(const FLAC__int32 residua
 		__m128i mm_res, mm_sum;
 
 		if(bps <= 16) {
-			FLAC__uint32 abs_residual_partition_sum;
 
 			for(partition = residual_sample = 0; partition < partitions; partition++) {
 				end += default_partition_samples;
-				abs_residual_partition_sum = 0;
 				mm_sum = _mm_setzero_si128();
 
 				e1 = (residual_sample + 3) & ~3; e3 = end & ~3;
@@ -71,25 +69,27 @@ void FLAC__precompute_partition_info_sums_intrin_ssse3(const FLAC__int32 residua
 					e1 = end; /* try flac -l 1 -b 16 and you'll be here */
 
 				/* assumption: residual[] is properly aligned so (residual + e1) is properly aligned too and _mm_loadu_si128() is fast */
-				for( ; residual_sample < e1; residual_sample++)
-					abs_residual_partition_sum += abs(residual[residual_sample]); /* abs(INT_MIN) is undefined, but if the residual is INT_MIN we have bigger problems */
+				for( ; residual_sample < e1; residual_sample++) {
+					mm_res = _mm_cvtsi32_si128(residual[residual_sample]);
+					mm_res = _mm_abs_epi32(mm_res); /* abs(INT_MIN) is undefined, but if the residual is INT_MIN we have bigger problems */
+					mm_sum = _mm_add_epi32(mm_sum, mm_res);
+				}
 
 				for( ; residual_sample < e3; residual_sample+=4) {
 					mm_res = _mm_loadu_si128((const __m128i*)(residual+residual_sample));
-
 					mm_res = _mm_abs_epi32(mm_res);
+					mm_sum = _mm_add_epi32(mm_sum, mm_res);
+				}
 
+				for( ; residual_sample < end; residual_sample++) {
+					mm_res = _mm_cvtsi32_si128(residual[residual_sample]);
+					mm_res = _mm_abs_epi32(mm_res);
 					mm_sum = _mm_add_epi32(mm_sum, mm_res);
 				}
 
 				mm_sum = _mm_hadd_epi32(mm_sum, mm_sum);
 				mm_sum = _mm_hadd_epi32(mm_sum, mm_sum);
-				abs_residual_partition_sum += _mm_cvtsi128_si32(mm_sum);
-
-				for( ; residual_sample < end; residual_sample++)
-					abs_residual_partition_sum += abs(residual[residual_sample]);
-
-				abs_residual_partition_sums[partition] = abs_residual_partition_sum;
+				abs_residual_partition_sums[partition] = _mm_cvtsi128_si32(mm_sum);
 			}
 		}
 		else { /* have to pessimistically use 64 bits for accumulator */
